@@ -1,150 +1,28 @@
+use strict;
+use warnings;
+
 package local::lib;
+
 use 5.006;
-BEGIN {
-  if ($ENV{RELEASE_TESTING}) {
-    require strict;
-    strict->import;
-    require warnings;
-    warnings->import;
-  }
-}
-use Config ();
 
-our $VERSION = '2.000_026';
-$VERSION =~ tr/_//d;
+use File::Spec ();
+use Config;
 
-BEGIN {
-  *_WIN32 = ($^O eq 'MSWin32' || $^O eq 'NetWare' || $^O eq 'symbian')
-    ? sub(){1} : sub(){0};
-  # punt on these systems
-  *_USE_FSPEC = ($^O eq 'MacOS' || $^O eq 'VMS' || $INC{'File/Spec.pm'})
-    ? sub(){1} : sub(){0};
-}
-my $_archname = $Config::Config{archname};
-my $_version = $Config::Config{version};
-my @_inc_version_list = reverse split / /, $Config::Config{inc_version_list};
-my $_path_sep = $Config::Config{path_sep};
-
-our $_DIR_JOIN = _WIN32 ? '\\' : '/';
-our $_DIR_SPLIT = (_WIN32 || $^O eq 'cygwin') ? qr{[\\/]}
-                                              : qr{/};
-our $_ROOT = _WIN32 ? do {
-  my $UNC = qr{[\\/]{2}[^\\/]+[\\/][^\\/]+};
-  qr{^(?:$UNC|[A-Za-z]:|)$_DIR_SPLIT};
-} : qr{^/};
-our $_PERL;
-
-sub _perl {
-  if (!$_PERL) {
-    # untaint and validate
-    ($_PERL, my $exe) = $^X =~ /((?:.*$_DIR_SPLIT)?(.+))/;
-    $_PERL = 'perl'
-      if $exe !~ /perl/;
-    if (_is_abs($_PERL)) {
-    }
-    elsif (-x $Config::Config{perlpath}) {
-      $_PERL = $Config::Config{perlpath};
-    }
-    elsif ($_PERL =~ $_DIR_SPLIT && -x $_PERL) {
-      $_PERL = _rel2abs($_PERL);
-    }
-    else {
-      ($_PERL) =
-        map { /(.*)/ }
-        grep { -x $_ }
-        map { ($_, _WIN32 ? ("$_.exe") : ()) }
-        map { join($_DIR_JOIN, $_, $_PERL) }
-        split /\Q$_path_sep\E/, $ENV{PATH};
-    }
-  }
-  $_PERL;
-}
-
-sub _cwd {
-  if (my $cwd
-    = defined &Cwd::sys_cwd ? \&Cwd::sys_cwd
-    : defined &Cwd::cwd     ? \&Cwd::cwd
-    : undef
-  ) {
-    no warnings 'redefine';
-    *_cwd = $cwd;
-    goto &$cwd;
-  }
-  my $drive = shift;
-  return Win32::GetCwd()
-    if _WIN32 && defined &Win32::GetCwd && !$drive;
-  local @ENV{qw(PATH IFS CDPATH ENV BASH_ENV)};
-  my $cmd = $drive ? "eval { Cwd::getdcwd(q($drive)) }"
-                   : 'getcwd';
-  my $perl = _perl;
-  my $cwd = `"$perl" -MCwd -le "print $cmd"`;
-  chomp $cwd;
-  if (!length $cwd && $drive) {
-    $cwd = $drive;
-  }
-  $cwd =~ s/$_DIR_SPLIT?$/$_DIR_JOIN/;
-  $cwd;
-}
-
-sub _catdir {
-  if (_USE_FSPEC) {
-    require File::Spec;
-    File::Spec->catdir(@_);
-  }
-  else {
-    my $dir = join($_DIR_JOIN, @_);
-    $dir =~ s{($_DIR_SPLIT)(?:\.?$_DIR_SPLIT)+}{$1}g;
-    $dir;
-  }
-}
-
-sub _is_abs {
-  if (_USE_FSPEC) {
-    require File::Spec;
-    File::Spec->file_name_is_absolute($_[0]);
-  }
-  else {
-    $_[0] =~ $_ROOT;
-  }
-}
-
-sub _rel2abs {
-  my ($dir, $base) = @_;
-  return $dir
-    if _is_abs($dir);
-
-  $base = _WIN32 && $dir =~ s/^([A-Za-z]:)// ? _cwd("$1")
-        : $base                              ? _rel2abs($base)
-                                             : _cwd;
-  return _catdir($base, $dir);
-}
-
-our $_DEVNULL;
-sub _devnull {
-  return $_DEVNULL ||=
-    _USE_FSPEC      ? (require File::Spec, File::Spec->devnull)
-    : _WIN32        ? 'nul'
-    : $^O eq 'os2'  ? '/dev/nul'
-    : '/dev/null';
-}
+our $VERSION = '2.000003'; # 2.0.3
+$VERSION = eval $VERSION;
 
 sub import {
   my ($class, @args) = @_;
-  if ($0 eq '-') {
-    push @args, @ARGV;
-    require Cwd;
-  }
 
   my @steps;
   my %opts;
-  my %attr;
   my $shelltype;
 
   while (@args) {
     my $arg = shift @args;
     # check for lethal dash first to stop processing before causing problems
     # the fancy dash is U+2212 or \xE2\x88\x92
-    if ($arg =~ /\xE2\x88\x92/) {
+    if ($arg =~ /\xE2\x88\x92/ or $arg =~ /−/) {
       die <<'DEATH';
 WHOA THERE! It looks like you've got some fancy dashes in your commandline!
 These are *not* the traditional -- dashes that software recognizes. You
@@ -180,24 +58,18 @@ DEATH
     elsif ( $arg eq '--no-create' ) {
       $opts{no_create} = 1;
     }
-    elsif ( $arg eq '--quiet' ) {
-      $attr{quiet} = 1;
-    }
-    elsif ( $arg eq '--always' ) {
-      $attr{always} = 1;
-    }
     elsif ( $arg =~ /^--/ ) {
       die "Unknown import argument: $arg";
     }
     else {
-      push @steps, ['activate', $arg, \%opts];
+      push @steps, ['activate', $arg];
     }
   }
   if (!@steps) {
-    push @steps, ['activate', undef, \%opts];
+    push @steps, ['activate', undef];
   }
 
-  my $self = $class->new(%attr);
+  my $self = $class->new(%opts);
 
   for (@steps) {
     my ($method, @args) = @$_;
@@ -228,7 +100,13 @@ sub libs { $_[0]->{libs}   ||= [ \'PERL5LIB' ] }
 sub bins { $_[0]->{bins}   ||= [ \'PATH' ] }
 sub roots { $_[0]->{roots} ||= [ \'PERL_LOCAL_LIB_ROOT' ] }
 sub extra { $_[0]->{extra} ||= {} }
-sub quiet { $_[0]->{quiet} }
+sub shelltype { $_[0]->{shelltype} ||= $_[0]->guess_shelltype }
+sub no_create { $_[0]->{no_create} }
+
+my $_archname = $Config{archname};
+my $_version  = $Config{version};
+my @_inc_version_list = reverse split / /, $Config{inc_version_list};
+my $_path_sep = $Config{path_sep};
 
 sub _as_list {
   my $list = shift;
@@ -251,34 +129,34 @@ my @_lib_subdirs = (
   [$_version, $_archname],
   [$_version],
   [$_archname],
-  (map [$_], @_inc_version_list),
+  (@_inc_version_list ? \@_inc_version_list : ()),
   [],
 );
 
 sub install_base_bin_path {
   my ($class, $path) = @_;
-  return _catdir($path, 'bin');
+  return File::Spec->catdir($path, 'bin');
 }
 sub install_base_perl_path {
   my ($class, $path) = @_;
-  return _catdir($path, 'lib', 'perl5');
+  return File::Spec->catdir($path, 'lib', 'perl5');
 }
 sub install_base_arch_path {
   my ($class, $path) = @_;
-  _catdir($class->install_base_perl_path($path), $_archname);
+  File::Spec->catdir($class->install_base_perl_path($path), $_archname);
 }
 
 sub lib_paths_for {
   my ($class, $path) = @_;
   my $base = $class->install_base_perl_path($path);
-  return map { _catdir($base, @$_) } @_lib_subdirs;
+  return map { File::Spec->catdir($base, @$_) } @_lib_subdirs;
 }
 
 sub _mm_escape_path {
   my $path = shift;
-  $path =~ s/\\/\\\\/g;
+  $path =~ s/\\/\\\\\\\\/g;
   if ($path =~ s/ /\\ /g) {
-    $path = qq{"$path"};
+    $path = qq{"\\"$path\\""};
   }
   return $path;
 }
@@ -364,12 +242,11 @@ sub deactivate_all {
 }
 
 sub activate {
-  my ($self, $path, $opts) = @_;
-  $opts ||= {};
+  my ($self, $path) = @_;
   $self = $self->new unless ref $self;
   $path = $self->resolve_path($path);
-  $self->ensure_dir_structure_for($path, { quiet => $self->quiet })
-    unless $opts->{no_create};
+  $self->ensure_dir_structure_for($path)
+    unless $self->no_create;
 
   $path = $self->normalize_path($path);
 
@@ -380,7 +257,7 @@ sub activate {
   }
 
   my %args;
-  if ($opts->{always} || !@active_lls || $active_lls[0] ne $path) {
+  if (!@active_lls || $active_lls[0] ne $path) {
     %args = (
       bins  => [ $self->install_base_bin_path($path), @{$self->bins} ],
       libs  => [ $self->install_base_perl_path($path), @{$self->libs} ],
@@ -402,19 +279,7 @@ sub normalize_path {
 }
 
 sub build_environment_vars_for {
-  my $self = $_[0]->new->activate($_[1], { always => 1 });
-  $self->build_environment_vars;
-}
-sub build_activate_environment_vars_for {
-  my $self = $_[0]->new->activate($_[1], { always => 1 });
-  $self->build_environment_vars;
-}
-sub build_deactivate_environment_vars_for {
-  my $self = $_[0]->new->deactivate($_[1]);
-  $self->build_environment_vars;
-}
-sub build_deact_all_environment_vars_for {
-  my $self = $_[0]->new->deactivate_all;
+  my $self = $_[0]->new->activate($_[1]);
   $self->build_environment_vars;
 }
 sub build_environment_vars {
@@ -434,12 +299,6 @@ sub setup_local_lib_for {
 
 sub setup_local_lib {
   my $self = shift;
-
-  # if Carp is already loaded, ensure Carp::Heavy is also loaded, to avoid
-  # $VERSION mismatch errors (Carp::Heavy loads Carp, so we do not need to
-  # check in the other direction)
-  require Carp::Heavy if $INC{'Carp.pm'};
-
   $self->setup_env_hash;
   @INC = @{$self->inc};
 }
@@ -466,7 +325,7 @@ sub print_environment_vars_for {
 }
 
 sub environment_vars_string_for {
-  my $self = $_[0]->new->activate($_[1], { always => 1});
+  my $self = $_[0]->new->activate($_[1]);
   $self->environment_vars_string;
 }
 sub environment_vars_string {
@@ -474,22 +333,14 @@ sub environment_vars_string {
 
   $shelltype ||= $self->guess_shelltype;
 
-  my $extra = $self->extra;
+  my $build_method = "build_${shelltype}_env_declaration";
+
   my @envs = (
     PATH                => $self->bins,
     PERL5LIB            => $self->libs,
     PERL_LOCAL_LIB_ROOT => $self->roots,
-    map { $_ => $extra->{$_} } sort keys %$extra,
+    %{$self->extra},
   );
-  $self->_build_env_string($shelltype, \@envs);
-}
-
-sub _build_env_string {
-  my ($self, $shelltype, $envs) = @_;
-  my @envs = @$envs;
-
-  my $build_method = "build_${shelltype}_env_declaration";
-
   my $out = '';
   while (@envs) {
     my ($name, $value) = (shift(@envs), shift(@envs));
@@ -501,9 +352,17 @@ sub _build_env_string {
         && ${$value->[0]} eq $name) {
       next;
     }
+    if (
+        !ref $value
+        and defined $value
+          ? (defined $ENV{$name} && $value eq $ENV{$name})
+          : !defined $ENV{$name}
+    ) {
+      next;
+    }
     $out .= $self->$build_method($name, $value);
   }
-  my $wrap_method = "wrap_${shelltype}_output";
+  my $wrap_method = 'wrap_' . $self->shelltype . '_output';
   if ($self->can($wrap_method)) {
     return $self->$wrap_method($out);
   }
@@ -512,21 +371,21 @@ sub _build_env_string {
 
 sub build_bourne_env_declaration {
   my ($class, $name, $args) = @_;
-  my $value = $class->_interpolate($args, '${%s:-}', qr/["\\\$!`]/, '\\%s');
+  my $value = $class->_interpolate($args, '$%s', '"', '\\%s');
 
   if (!defined $value) {
     return qq{unset $name;\n};
   }
 
-  $value =~ s/(^|\G|$_path_sep)\$\{$name:-\}$_path_sep/$1\${$name}\${$name:+$_path_sep}/g;
-  $value =~ s/$_path_sep\$\{$name:-\}$/\${$name:+$_path_sep\${$name}}/;
+  $value =~ s/(^|\G|$_path_sep)\$$name$_path_sep/$1\$$name\${$name+$_path_sep}/g;
+  $value =~ s/$_path_sep\$$name$/\${$name+$_path_sep}\$$name/;
 
-  qq{${name}="$value"; export ${name};\n}
+  qq{${name}="$value";\nexport ${name};\n}
 }
 
 sub build_csh_env_declaration {
   my ($class, $name, $args) = @_;
-  my ($value, @vars) = $class->_interpolate($args, '${%s}', qr/["\$]/, '"\\%s"');
+  my ($value, @vars) = $class->_interpolate($args, '$%s', '"', '"\\%s"');
   if (!defined $value) {
     return qq{unsetenv $name;\n};
   }
@@ -537,9 +396,9 @@ sub build_csh_env_declaration {
   }
 
   my $value_without = $value;
-  if ($value_without =~ s/(?:^|$_path_sep)\$\{$name\}(?:$_path_sep|$)//g) {
-    $out .= qq{if "\${$name}" != '' setenv $name "$value";\n};
-    $out .= qq{if "\${$name}" == '' };
+  if ($value_without =~ s/(?:^|$_path_sep)\$$name(?:$_path_sep|$)//g) {
+    $out .= qq{if "\$$name" != '' setenv $name "$value";\n};
+    $out .= qq{if "\$$name" == '' };
   }
   $out .= qq{setenv $name "$value_without";\n};
   return $out;
@@ -547,7 +406,7 @@ sub build_csh_env_declaration {
 
 sub build_cmd_env_declaration {
   my ($class, $name, $args) = @_;
-  my $value = $class->_interpolate($args, '%%%s%%', qr(%), '%s');
+  my $value = $class->_interpolate($args, '%%%s%%', qr([()!^"<>&|]), '^%s');
   if (!$value) {
     return qq{\@set $name=\n};
   }
@@ -555,19 +414,19 @@ sub build_cmd_env_declaration {
   my $out = '';
   my $value_without = $value;
   if ($value_without =~ s/(?:^|$_path_sep)%$name%(?:$_path_sep|$)//g) {
-    $out .= qq{\@if not "%$name%"=="" set "$name=$value"\n};
+    $out .= qq{\@if not "%$name%"=="" set $name=$value\n};
     $out .= qq{\@if "%$name%"=="" };
   }
-  $out .= qq{\@set "$name=$value_without"\n};
+  $out .= qq{\@set $name=$value_without\n};
   return $out;
 }
 
 sub build_powershell_env_declaration {
   my ($class, $name, $args) = @_;
-  my $value = $class->_interpolate($args, '$env:%s', qr/["\$]/, '`%s');
+  my $value = $class->_interpolate($args, '$env:%s', '"', '`%s');
 
   if (!$value) {
-    return qq{Remove-Item -ErrorAction 0 Env:\\$name;\n};
+    return qq{Remove-Item Env:\\$name;\n};
   }
 
   my $maybe_path_sep = qq{\$(if("\$env:$name"-eq""){""}else{"$_path_sep"})};
@@ -581,33 +440,6 @@ sub wrap_powershell_output {
   return $out || " \n";
 }
 
-sub build_fish_env_declaration {
-  my ($class, $name, $args) = @_;
-  my $value = $class->_interpolate($args, '$%s', qr/[\\"'$ ]/, '\\%s');
-  if (!defined $value) {
-    return qq{set -e $name;\n};
-  }
-
-  # fish has special handling for PATH, CDPATH, and MANPATH.  They are always
-  # treated as arrays, and joined with ; when storing the environment.  Other
-  # env vars can be arrays, but will be joined without a separator.  We only
-  # really care about PATH, but might as well make this routine more general.
-  if ($name =~ /^(?:CD|MAN)?PATH$/) {
-    $value =~ s/$_path_sep/ /g;
-    my $silent = $name =~ /^(?:CD)?PATH$/ ? " 2>"._devnull : '';
-    return qq{set -x $name $value$silent;\n};
-  }
-
-  my $out = '';
-  my $value_without = $value;
-  if ($value_without =~ s/(?:^|$_path_sep)\$$name(?:$_path_sep|$)//g) {
-    $out .= qq{set -q $name; and set -x $name $value;\n};
-    $out .= qq{set -q $name; or };
-  }
-  $out .= qq{set -x $name $value_without;\n};
-  $out;
-}
-
 sub _interpolate {
   my ($class, $args, $var_pat, $escape, $escape_pat) = @_;
   return
@@ -643,6 +475,20 @@ sub pipeline {
   }
 }
 
+=begin testing
+
+#:: test pipeline
+
+package local::lib;
+
+{ package Foo; sub foo { -$_[1] } sub bar { $_[1]+2 } sub baz { $_[1]+3 } }
+my $foo = bless({}, 'Foo');
+Test::More::ok($foo->${pipeline qw(foo bar baz)}(10) == -15);
+
+=end testing
+
+=cut
+
 sub resolve_path {
   my ($class, $path) = @_;
 
@@ -664,13 +510,32 @@ sub resolve_empty_path {
   }
 }
 
+=begin testing
+
+#:: test classmethod setup
+
+my $c = 'local::lib';
+
+=end testing
+
+=begin testing
+
+#:: test classmethod
+
+is($c->resolve_empty_path, '~/perl5');
+is($c->resolve_empty_path('foo'), 'foo');
+
+=end testing
+
+=cut
+
 sub resolve_home_path {
   my ($class, $path) = @_;
-  $path =~ /^~([^\/]*)/ or return $path;
-  my $user = $1;
+  return $path unless ($path =~ /^~/);
+  my ($user) = ($path =~ /^~([^\/]+)/); # can assume ^~ so undef for 'us'
   my $homedir = do {
-    if (! length($user) && defined $ENV{HOME}) {
-      $ENV{HOME};
+    if (!defined $user && defined $ENV{HOME}) {
+      $ENV{HOME}
     }
     else {
       require File::Glob;
@@ -678,7 +543,7 @@ sub resolve_home_path {
     }
   };
   unless (defined $homedir) {
-    require Carp; require Carp::Heavy;
+    require Carp;
     Carp::croak(
       "Couldn't resolve homedir for "
       .(defined $user ? $user : 'current user')
@@ -690,61 +555,69 @@ sub resolve_home_path {
 
 sub resolve_relative_path {
   my ($class, $path) = @_;
-  _rel2abs($path);
+  $path = File::Spec->rel2abs($path);
 }
 
+=begin testing
+
+#:: test classmethod
+
+local *File::Spec::rel2abs = sub { shift; 'FOO'.shift; };
+is($c->resolve_relative_path('bar'),'FOObar');
+
+=end testing
+
+=cut
+
 sub ensure_dir_structure_for {
-  my ($class, $path, $opts) = @_;
-  $opts ||= {};
+  my ($class, $path) = @_;
+  unless (-d $path) {
+    warn "Attempting to create directory ${path}\n";
+  }
+  require File::Basename;
   my @dirs;
-  foreach my $dir (
-    $class->lib_paths_for($path),
-    $class->install_base_bin_path($path),
-  ) {
-    my $d = $dir;
-    while (!-d $d) {
-      push @dirs, $d;
-      require File::Basename;
-      $d = File::Basename::dirname($d);
-    }
-  }
-
-  warn "Attempting to create directory ${path}\n"
-    if !$opts->{quiet} && @dirs;
-
-  my %seen;
-  foreach my $dir (reverse @dirs) {
-    next
-      if $seen{$dir}++;
-
-    mkdir $dir
-      or -d $dir
-      or die "Unable to create $dir: $!"
+  while(!-d $path) {
+    push @dirs, $path;
+    $path = File::Basename::dirname($path);
   }
+  mkdir $_ for reverse @dirs;
   return;
 }
 
+=begin testing
+
+#:: test classmethod
+
+File::Path::rmtree('t/var/splat');
+
+$c->ensure_dir_structure_for('t/var/splat');
+
+ok(-d 't/var/splat');
+
+=end testing
+
+=cut
+
 sub guess_shelltype {
   my $shellbin
-    = defined $ENV{SHELL} && length $ENV{SHELL}
-      ? ($ENV{SHELL} =~ /([\w.]+)$/)[-1]
+    = defined $ENV{SHELL}
+      ? (File::Spec->splitpath($ENV{SHELL}))[-1]
     : ( $^O eq 'MSWin32' && exists $ENV{'!EXITCODE'} )
       ? 'bash'
     : ( $^O eq 'MSWin32' && $ENV{PROMPT} && $ENV{COMSPEC} )
-      ? ($ENV{COMSPEC} =~ /([\w.]+)$/)[-1]
+      ? (File::Spec->splitpath($ENV{COMSPEC}))[-1]
     : ( $^O eq 'MSWin32' && !$ENV{PROMPT} )
       ? 'powershell.exe'
     : 'sh';
 
   for ($shellbin) {
     return
-        /csh$/                   ? 'csh'
-      : /fish$/                  ? 'fish'
-      : /command(?:\.com)?$/i    ? 'cmd'
-      : /cmd(?:\.exe)?$/i        ? 'cmd'
-      : /4nt(?:\.exe)?$/i        ? 'cmd'
-      : /powershell(?:\.exe)?$/i ? 'powershell'
-                                 : 'bourne';
+        /csh/              ? 'csh'
+      : /command\.com/i    ? 'cmd'
+      : /cmd\.exe/i        ? 'cmd'
+      : /4nt\.exe/i        ? 'cmd'
+      : /powershell\.exe/i ? 'powershell'
+                           : 'bourne';
   }
 }
 
@@ -776,15 +649,14 @@ From the shell -
 
   # Just print out useful shell commands
   $ perl -Mlocal::lib
-  PERL_MB_OPT='--install_base /home/username/perl5'; export PERL_MB_OPT;
-  PERL_MM_OPT='INSTALL_BASE=/home/username/perl5'; export PERL_MM_OPT;
-  PERL5LIB="/home/username/perl5/lib/perl5"; export PERL5LIB;
-  PATH="/home/username/perl5/bin:$PATH"; export PATH;
-  PERL_LOCAL_LIB_ROOT="/home/usename/perl5:$PERL_LOCAL_LIB_ROOT"; export PERL_LOCAL_LIB_ROOT;
+  export PERL_MB_OPT='--install_base /home/username/perl5'
+  export PERL_MM_OPT='INSTALL_BASE=/home/username/perl5'
+  export PERL5LIB="/home/username/perl5/lib/perl5"
+  export PATH="/home/username/perl5/bin:$PATH"
 
-From a F<.bash_profile> or F<.bashrc> file -
+From a .bashrc file -
 
-  eval "$(perl -I$HOME/perl5/lib/perl5 -Mlocal::lib)"
+  [ $SHLVL -eq 1 ] && eval "$(perl -I$HOME/perl5/lib/perl5 -Mlocal::lib)"
 
 =head2 The bootstrapping technique
 
@@ -820,6 +692,11 @@ Run this:
 If the system asks you whether it should automatically configure as much
 as possible, you would typically answer yes.
 
+In order to install local::lib into a directory other than the default, you need
+to specify the name of the directory when you call bootstrap, as follows:
+
+  perl Makefile.PL --bootstrap=~/foo
+
 =item 3.
 
 Run this: (local::lib assumes you have make installed on your system)
@@ -833,13 +710,20 @@ starts using our newly generated lib/ directory. If you are using bash or
 any other Bourne shells, you can add this to your shell startup script this
 way:
 
-  echo 'eval "$(perl -I$HOME/perl5/lib/perl5 -Mlocal::lib)"' >>~/.bashrc
+  echo '[ $SHLVL -eq 1 ] && eval "$(perl -I$HOME/perl5/lib/perl5 -Mlocal::lib)"' >>~/.bashrc
 
 If you are using C shell, you can do this as follows:
 
-  % echo $SHELL
   /bin/csh
-  $ echo 'eval `perl -I$HOME/perl5/lib/perl5 -Mlocal::lib`' >> ~/.cshrc
+  echo $SHELL
+  /bin/csh
+  echo 'eval `perl -I$HOME/perl5/lib/perl5 -Mlocal::lib`' >> ~/.cshrc
+
+If you passed to bootstrap a directory other than default, you also need to
+give that as import parameter to the call of the local::lib module like this
+way:
+
+  echo '[ $SHLVL -eq 1 ] && eval "$(perl -I$HOME/foo/lib/perl5 -Mlocal::lib=$HOME/foo)"' >>~/.bashrc
 
 After writing your shell configuration file, be sure to re-read it to get the
 changed settings into your current shell's environment. Bourne shells use
@@ -847,20 +731,6 @@ C<. ~/.bashrc> for this, whereas C shells use C<source ~/.cshrc>.
 
 =back
 
-=head3 Bootstrapping into an alternate directory
-
-In order to install local::lib into a directory other than the default, you need
-to specify the name of the directory when you call bootstrap.  Then, when
-setting up the environment variables, both perl and local::lib must be told the
-location of the bootstrap directory.  The setup process would look as follows:
-
-  perl Makefile.PL --bootstrap=~/foo
-  make test && make install
-  echo 'eval "$(perl -I$HOME/foo/lib/perl5 -Mlocal::lib=$HOME/foo)"' >>~/.bashrc
-  . ~/.bashrc
-
-=head3 Other bootstrapping options
-
 If you're on a slower machine, or are operating under draconian disk space
 limitations, you can disable the automatic generation of manpages from POD when
 installing modules by using the C<--no-manpages> argument when bootstrapping:
@@ -880,13 +750,11 @@ installation to install modules in different directories directly this way:
   cd ../mydir2
   ... REPEAT ...
 
-If you use F<.bashrc> to activate a local::lib automatically, the local::lib
-will be re-enabled in any sub-shells used, overriding adjustments you may have
-made in the parent shell.  To avoid this, you can initialize the local::lib in
-F<.bash_profile> rather than F<.bashrc>, or protect the local::lib invocation
-with a C<$SHLVL> check:
-
-  [ $SHLVL -eq 1 ] && eval "$(perl -I$HOME/perl5/lib/perl5 -Mlocal::lib)"
+When used in a C<.bashrc> file, it is recommended that you protect against
+re-activating a directory in a sub-shell.  This can be done by checking the
+C<$SHLVL> variable as shown in synopsis.  Without this, sub-shells created by
+the user or other programs will override changes made to the parent shell's
+environment.
 
 If you are working with several C<local::lib> environments, you may want to
 remove some of them from the current environment without disturbing the others.
@@ -1031,15 +899,6 @@ was added by C<local::lib>, instead of adding it.
 Remove all directories that were added to search paths by C<local::lib> from the
 search paths.
 
-=head2 --quiet
-
-Don't output any messages about directories being created.
-
-=head2 --always
-
-Always add directories to environment variables, ignoring if they are already
-included.
-
 =head2 --shelltype
 
 Specify the shell type to use for output.  By default, the shell will be
@@ -1063,8 +922,8 @@ likely to cause issues on Win32 systems.
 
 =back
 
-Attempts to create a local::lib directory, including subdirectories and all
-required parent directories. Throws an exception on failure.
+Attempts to create the given path, and all required parent directories. Throws
+an exception on failure.
 
 =head2 print_environment_vars_for
 
@@ -1196,8 +1055,8 @@ installation. Defaults to C<~/perl5>.
 
 =back
 
-Attempts to find the user's home directory.
-If no definite answer is available, throws an exception.
+Attempts to find the user's home directory. If installed, uses C<File::HomeDir>
+for this purpose. If no definite answer is available, throws an exception.
 
 =head2 resolve_relative_path
 
@@ -1395,10 +1254,11 @@ not set, a Bourne-compatible shell is assumed.
 
 =item * Should probably auto-fixup CPAN config if not already done.
 
-=item * On VMS and MacOS Classic (pre-OS X), local::lib loads L<File::Spec>.
-This means any L<File::Spec> version installed in the local::lib will be
-ignored by scripts using local::lib.  A workaround for this is using
-C<use lib "$local_lib/lib/perl5";> instead of using C<local::lib> directly.
+=item * local::lib loads L<File::Spec>.  When used to set shell variables,
+this isn't a problem.  When used inside a perl script, any L<File::Spec>
+version inside the local::lib will be ignored.  A workaround for this is using
+C<use lib "$ENV{HOME}/perl5/lib/perl5";> inside the script instead of using
+C<local::lib> directly.
 
 =item * Conflicts with L<ExtUtils::MakeMaker>'s C<PREFIX> option.
 C<local::lib> uses the C<INSTALL_BASE> option, as it has more predictable and
@@ -1466,7 +1326,7 @@ On Win32 systems, C<COMSPEC> is also examined.
 
 IRC:
 
-    Join #toolchain on irc.perl.org.
+    Join #local-lib on irc.perl.org.
 
 =head1 AUTHOR
 
